/* This file is Copyright 2000-2009 Meyer Sound Laboratories Inc.  See the included LICENSE.txt file for details. */  

#include <stdio.h>
#include "system/SetupSystem.h"
#include "util/MiscUtilityFunctions.h"

using namespace muscle;

static float GetDiffHours(uint64 t1, uint64 t2) {return (float) ((double)((int64)t1 - (int64) t2))/(60.0 * 60.0 * 1000000.0);}

// This program is used to test out muscle's time/date interpretation functions
int main(int argc, char ** argv) 
{
   CompleteSetupSystem css;

   Message argsMsg;
   ParseArgs(argc, argv, argsMsg);
   HandleStandardDaemonArgs(argsMsg);

   const uint64 epoch = 0;
   printf("epoch time (UTC) = %s\n", GetHumanReadableTimeString(epoch, MUSCLE_TIMEZONE_UTC)());
   printf("epoch time (loc) = %s\n", GetHumanReadableTimeString(epoch, MUSCLE_TIMEZONE_LOCAL)());

   const uint64 nowLocal = GetCurrentTime64(MUSCLE_TIMEZONE_LOCAL);
   const String nowLocalStr = GetHumanReadableTimeString(nowLocal, MUSCLE_TIMEZONE_LOCAL);
   printf("NOW (Local) = " UINT64_FORMAT_SPEC " = %s\n", nowLocal, nowLocalStr());
   const uint64 reparsedLocal = ParseHumanReadableTimeString(nowLocalStr(), MUSCLE_TIMEZONE_LOCAL);
   printf("   reparsed = " UINT64_FORMAT_SPEC " (diff=%.1f hours)\n", reparsedLocal, GetDiffHours(reparsedLocal, nowLocal));

   const uint64 nowUTC = GetCurrentTime64(MUSCLE_TIMEZONE_UTC);
   const String nowUTCStr = GetHumanReadableTimeString(nowUTC, MUSCLE_TIMEZONE_LOCAL);
   printf("NOW (UTC)   = " UINT64_FORMAT_SPEC " = %s\n            (or, in local terms, %s)\n", nowUTC, nowUTCStr(), GetHumanReadableTimeString(nowUTC, MUSCLE_TIMEZONE_UTC)());

   const uint64 reparsedUTC = ParseHumanReadableTimeString(nowUTCStr(), MUSCLE_TIMEZONE_LOCAL);
   printf("   reparsed = " UINT64_FORMAT_SPEC " (diff=%.1f hours)\n", reparsedUTC, GetDiffHours(reparsedUTC, nowUTC));

   printf("The offset between local time and UTC is %.1f hours.\n", GetDiffHours(nowLocal, nowUTC));

   HumanReadableTimeValues v;
   if (GetHumanReadableTimeValues(nowLocal, v, MUSCLE_TIMEZONE_LOCAL) == B_NO_ERROR) printf("HRTV(local) = [%s]\n", v.ExpandTokens("%T DoW=%w (%t) (%f) (%q) (micro=%x) (rand=%r)")());
                                                                                else printf("Error getting human readable time values for local!\n");
   if (GetHumanReadableTimeValues(nowUTC, v, MUSCLE_TIMEZONE_LOCAL) == B_NO_ERROR)   printf("HRTV(UTC)   = [%s]\n", v.ExpandTokens("%T DoW=%w (%t) (%f) (%q) (micro=%x) (rand=%r)")());
                                                                                else printf("Error getting human readable time values for UTC!\n");
   return 0;
}
